#!/usr/bin/python
"""csv2netcdf

Usage:
  csv2netcdf --file FILE [--overwrite] [--group GROUP]
  csv2netcdf (-h | --help)
  csv2netcdf --version

Options:
  -h --help     Show this screen
  --version     Show version
"""
import sys
import netCDF4
from docopt import docopt
from datetime import datetime, timedelta
import logging

DEBUG = True
VERSION = "0.0.1"

if DEBUG:
    logging.basicConfig(filename='/tmp/debug.log',level=logging.DEBUG)

def main(args):

    _file = args["FILE"]
    wr = args["--overwrite"]
    group_mode = args["--group"]
    group_name = args["GROUP"]

    #filename = ".".join([_file.split(".")[0], "nc"])
    filename = _file

    if wr:
        nc = netCDF4.Dataset(filename, 'w')
    else:
        nc = netCDF4.Dataset(filename, 'a')

    if group_mode:
        _grp = nc.createGroup(group_name)
    else:
        _grp = nc
        
    time = _grp.createDimension('time', None)
    lineno = 0
    cnt = 0
    tmp = {}
    for line in sys.stdin:
        logging.debug(line)
        a = line.split(",")
        for i, item in enumerate(a):
            var = item.strip()
            if var == "Date/Time":
                if lineno == 0:
                    _grp.createVariable("time", 'f8', ('time',))
                    tmp[i] = "time"
                else:
                    value = var
                    key = tmp[i]
                    cnt = lineno - 1
                    _grp.variables[key][cnt] = value
            else:
                if lineno == 0:
                    _grp.createVariable(var, 'f8', ('time',))
                    tmp[i] = var
                else:
                    if i == 0:
                        dt = datetime.strptime(var, "%d-%b-%y %H:%M:%S.%f")
                        value = totimestamp(dt)
                    else:
                        value = float(var)
                    key = tmp[i]
                    cnt = lineno - 1
                    _grp.variables[key][cnt] = value
        lineno+= 1
            
def totimestamp(dt, epoch=datetime(1970,1,1)):
    td = dt - epoch
    # return td.total_seconds()
    output = (td.microseconds + (td.seconds + td.days * 24 * 3600) * 10**6) / 1e6
    return "%.6f" % output

if __name__ == '__main__':
    arguments = docopt(__doc__, version=VERSION)
    if DEBUG:
        logging.debug(arguments)
    main(arguments)
