<?php

require_once($ADEI_ROOTDIR . "/classes/draw.php");

abstract class BASETimeSeriesView extends VIEW {
 var $axis_label_font_size = 8;
 var $axis_title_font_size = 8;
 var $axis_title_symbol_width = 8;
 var $axis_title_height = 20;
 var $x_axis_margin_correction = 10;


 function __construct(REQUEST $req  = NULL, $options) {
    parent::__construct($req, $options);
 }

 function GetTimeAxisLabelsWidth($format) {
    global $GRAPH_MARGINS;

    //    $bottom_margin_threshold = $length > 86400 && $length < 1036800 ? 70 : 40;
    if ($format['time_format']) {
	$formatted = date($format['time_format']);
	$symbols = strlen($formatted);
	$ret = $this->axis_title_symbol_width * $symbols;
    } else {
	$ret = $GRAPH_MARGINS['axis'] - $this->axis_title_height;
    }
    return $ret;
 }

 function GetYAxisLabelsWidth($format) {
    global $GRAPH_MARGINS;

    return $GRAPH_MARGINS['axis'] - $this->axis_title_height;
 }


 private function GetTimeAxisMajorTickStep($length, $width, $min_tick) {
    $n_ticks = $width / 40;
    if (!$n_ticks) $n_ticks = 1;
    $tick_step = $length / $n_ticks;
    return 2 * $min_tick * ceil($tick_step / $min_tick / 2);
 }

 function GetTimeAxisFormat(INTERVAL $ivl, $width = 0) {
     global $GRAPH_SUBSECOND_THRESHOLD;

    $time_format = false;
    $min_ticks = 0; $maj_ticks = 0;

    $length = $ivl->GetWindowSize();

    if ($length > 315360000) { // 10 years
        $time_format = 'Y';
        $min_ticks = 315360000;
    } elseif ($length > 31104000) { // 1 year
        $time_format = 'M, Y';
        $min_ticks = 3104000;
    } elseif ($length > 1036800) { // 12 days
        $time_format = 'M d';
        $min_ticks = 3600 * 24;
    } elseif ($length > 86400) { // 1 day
        $time_format = 'M d, H:i';
        $min_ticks = 3600 * 4;
    } elseif ($length > 14400) { // 4 hours
        $time_format = 'H:i';
        $min_ticks = 1800;
    } elseif ($length > 1800) { // 30 min
        $time_format = 'H:i';
        $min_ticks = 150;
    } elseif ($length > 600) { // 5 min
        $time_format = 'H:i';
        $min_ticks = 30;
    } elseif ($length > $GRAPH_SUBSECOND_THRESHOLD) {
        $time_format = 'H:i:s';
        $min_ticks = $GRAPH_SUBSECOND_THRESHOLD;
    } else { 
	/* We need to adjust data in the VIEW code, like this:
	    $from = $xdata[0];
	    foreach ($xdata as &$x) $x = dsMathPreciseSubstract($x, $from);*/
    }

    if (($width)&&($min_ticks)) {
        $maj_ticks = $this->GetTimeAxisMajorTickStep($length, $width, $min_ticks);
    }

    return array(
	'time_format' => $time_format,
	'minor_tick_step' => $min_ticks,
	'major_tick_step' => $maj_ticks
    );
 }

 function ConfigureTimeAxis(Graph $graph, INTERVAL $ivl, $width) {
     global $GRAPH_SUBSECOND_THRESHOLD;

    $format = $this->GetTimeAxisFormat($ivl, $width);
    if ($format['time_format']) {
        $graph->SetScale('datlin', 0, 0, $this->ivl->GetWindowStart(), $this->ivl->GetWindowEnd());
	$graph->xaxis->scale->SetDateFormat($format['time_format']);
    } else {
        $graph->SetScale('linlin', 0, 0, 0, $this->ivl->GetWindowSize());
	$graph->xaxis->SetLabelFormatCallback(array(new DRAW_JPGraphAxisHolder(NULL, NULL, $graph->xaxis), 'YLabelFormat'));
    }

    if ($format['major_tick_step'])
	$graph->xaxis->scale->ticks->Set($format['major_tick_step'], $format['minor_tick_step']);

    $graph->xaxis->SetPos('min');
    $graph->xaxis->SetLabelAngle(90);
    $graph->xaxis->SetTextLabelInterval(1);
    $graph->xaxis->SetFont(FF_ARIAL, FS_NORMAL, $this->axis_label_font_size);
    $graph->xaxis->title->SetFont(FF_ARIAL, FS_BOLD, $this->axis_title_font_size);

    $format['labels_width'] = $this->GetTimeAxisLabelsWidth($format);
    return $format;
 }

 function GetYAxisFormat(array $minmax = NULL, $height = 0) {
    return array();
 }

 function ConfigureYAxis(Graph $graph, Axis $axis, array $minmax = NULL, $height = 0) {
    $axis->SetLabelFormatCallback(array(new DRAW_JPGraphAxisHolder(NULL, NULL, $axis), 'YLabelFormat'));

    $axis->SetFont(FF_ARIAL, FS_NORMAL, $this->axis_label_font_size);
    $axis->title->SetFont(FF_ARIAL, FS_BOLD, $this->axis_title_font_size);

    $format = $this->GetYAxisFormat($minmax, $height);
    $format['labels_width'] = $this->GetYAxisLabelsWidth($format);
    return $format;
 }

 function ConfigureXAxisTitle(Graph $graph, $label, $position = 0) {
    $graph->xaxis->SetTitle("$label");

    if ($position) {
	$graph->xaxis->SetTitleMargin($position - $this->x_axis_margin_correction);
	return $position + $this->axis_title_height;// + $this->x_axis_margin_correction;
    }

    return false;
 }

 function ConfigureYAxisTitle(Graph $graph, Axis $axis, $label, $position = 0) {
    $axis->SetTitle("$label");

    if ($position) {
	$axis->SetTitleMargin($position);
	return $position + $this->axis_title_height;
    }

    return false;
 }

}

?>