<?php
global $ADEI_ROOTDIR;
require_once($ADEI_ROOTDIR . "/classes/views/basetimeseriesview.php");

abstract class AggregationMethods
{
    const SkipMissing = "skip";
    const UseLast = "uselast";

    static function GetSelectOptions()
    {
        return array(
            array("value" => self::SkipMissing, "label" => "Skip Where Missing"),
            array("value" => self::UseLast, "label" => "Use Last Available Value"),
        );
    }
}

class CALCView extends BASETimeSeriesView
{
    function __construct(REQUEST $req = NULL, $options)
    {
        parent::__construct($req, $options);
    }

    function GetOptions()
    {
        global $GRAPH_MARGINS;
        $chan = $this->GetDataVariants(array("x", "y"));

        $req = $this->req->CreateDataRequest();
        
        $ymargin = $this->GetYAxisLabelsWidth($this->GetYAxisFormat());
        $xmargin = $this->GetTimeAxisLabelsWidth($this->GetTimeAxisFormat($this->ivl));
        
        $ylabel_margin = $req->GetProp("view_ylabelmargin", 0);
        $xlabel_margin = $req->GetProp("view_xlabelmargin", 0);

        return array(
            array("select" => array("label" => _("X"), "id" => "x", "options" => $chan[0])),
            array("xml" => "<br/>"),
            array("select" => array("label" => _("Y"), "id" => "y", "options" => $chan[1])),
            array("xml" => "<br/>"),
            array("select" => array("label" => _("Aggregation Method"), "id" => "channel_aggregation", "options" => RESAMPLER::GetSelectList())),
            array("xml" => "<br/>"),
            array("input" => array("label" => _("Function"), "id" => "math_func", "type" => "text", "size" => "20", "value" => "")),
            array("xml" => "<br/>"),
            array("input" => array("label" => _("X-axis Title"), "id" => "xlabel", "type" => "text", "size" => "8", "value" => "")),
            array("input" => array("label" => _(" Offset ($xmargin) correction"), "id" => "xlabelmargin", "type" => "text", "size" => "8", "value" => "$xlabel_margin")),
            array("xml" => "<br/>"),
            array("input" => array("label" => _("Y-axis Title"), "id" => "ylabel", "type" => "text", "size" => "8", "value" => "")),
            array("input" => array("label" => _(" Offset ($ymargin) correction"), "id" => "ylabelmargin", "type" => "text", "size" => "8", "value" => "$ylabel_margin")),
            array("xml" => "<br/>"),

        );
    }

    function GetView()
    {
        $req = $this->req->CreateDataRequest();
        $agg = $req->GetProp("view_channel_aggregation", false);
        $this->math_func = $req->GetProp("view_math_func", false);
        $x = $req->GetProp("view_x", false);
        $y = $req->GetProp("view_y", false);

        if (!$x) throw new ADEIException(translate("Parameters view_x are not set"));
        if (!$y) throw new ADEIException(translate("Parameters view_y are not set"));
        if (!$this->math_func) throw new ADEIException(translate("Arithmetic operation is not specified"));

        $func = dsMathCreateFunction('$x,$y', $this->math_func);
        if (!$func) throw new ADEIException(translate("Can't parse function %s", $this->math_func));

        $res_full = $this->GetData(array("x", "y"), NULL, VIEW::GET_RAW_DATA);
        $total = sizeof($res_full['data']);
	$res = $this->ResampleData($res_full, $agg);
	$missing = $res['info']['incomplete'];
	$displayed = sizeof($res['data']);
	
        $data = array();
        foreach ($res['data'] as $t => $v) {
            $point = $func($v[0], $v[1]);
            $data[$t] = $point;
	}
        ksort($data);

        $el_parent_x = '';
        $el_x = '';
        $el_parent_y = '';
        $el_y = '';
        $arr = $this->GetOptions();
        foreach ($arr[0]['select']['options'] as $x_key => $x_lv) {
            $el = $arr[0]['select']['options'][$x_key];
            if ($el["disabled"] == 1) {
                $el_parent_x = $el['label'];
            } elseif ($el['value'] == $x) {
                $el_x = $el['label'];
                break;
            }
        }

        foreach ($arr[2]['select']['options'] as $y_key => $y_lv) {
            $el = $arr[2]['select']['options'][$y_key];
            if ($el["disabled"] == 1) {
                $el_parent_y = $el['label'];
            } elseif ($el['value'] == $y) {
                $el_y = $el["label"];
                break;
            }
        }

        $info = array(
            array("title" => _("Acquired Data Records"), "value" => "$total (of $this->max_points)"),
            array("title" => _("Skipped Incomplete Records"), "value" => "$missing"),
            array("title" => _("Displayed Records"), "value" => "$displayed"),
            array("title" => _("Operation"), "value" => "$this->math_func"),
            array("title" => _("x"), "value" => "$el_parent_x - $el_x"),
            array("title" => _("y"), "value" => "$el_parent_y - $el_y"),
        );

        if (sizeof($data) == 0) {
            return array("info" => $info);
        }

        $from = date("D, d M Y H:i:s", key($data));
        end($data);
        $to = date("D, d M Y H:i:s", key($data));
        reset($data);
        $info = array_merge($info, array(
            array("title" => _("From"), "value" => "$from"),
            array("title" => _("To"), "value" => "$to")
        ));

        $view = $this->GetGraphView($data, $req);
        $view["info"] = $info;
        return $view;
    }

    private function GetGraphView($data, $req)
    {
        global $GRAPH_MARGINS;
        global $TMP_PATH;
        global $GRAPH_SUBSECOND_THRESHOLD;

        $ydata = array_values($data);
        $xdata = array_keys($data);

        $tmp_file = ADEI::GetTmpFile();
        $x = $req->GetProp("view_x", false);
        $y = $req->GetProp("view_y", false);

        $graph = new Graph($this->width, $this->height);
	$time_axis_format = $this->ConfigureTimeAxis($graph, $this->ivl, $this->width);
	$y_axis_format = $this->ConfigureYAxis($graph, $graph->yaxis, array(min($ydata), max($ydata)), $this->height);

//        $ydata = array_merge(array(''), $ydata, array(''));
//        $xdata = array_merge(array($this->ivl->GetWindowStart()), $xdata, array($this->ivl->GetWindowEnd()));

	if ($time_axis_format['time_format'] === false) {
	    $from = $xdata[0];
	    foreach ($xdata as &$x) $x = dsMathPreciseSubstract($x, $from);
	}

        $xlabel = $req->GetProp("view_xlabel", false);
	$xlabel_margin = $time_axis_format['labels_width'] + $req->GetProp("view_xlabelmargin", 0);
        if ($xlabel) $xlabel_margin = $this->ConfigureXAxisTitle($graph, $xlabel, $xlabel_margin);

	$ylabel = $req->GetProp("view_ylabel", false);
        $ylabel_margin = $y_axis_format['labels_width'] + $req->GetProp("view_ylabelmargin", 0);
	if ($ylabel) $ylabel_margin = $this->ConfigureYAxisTitle($graph, $graph->yaxis, $ylabel, $ylabel_margin);

        //$graph->title->Set("$this->math_func");
        $graph->img->SetMargin($ylabel_margin, 10, 20, $xlabel_margin + 10);

        $lineplot = new LinePlot($ydata, $xdata);
        $graph->Add($lineplot);
        $graph->Stroke("$TMP_PATH/$tmp_file");

        return array(
            "img" => array("id" => $tmp_file)
        );
    }

};
