#! /bin/bash

. opts.sh

[ -z "$setup" ] && { echo "Setup must be specified (use 'all' to populate all known setups)"; exit 1; }

case $setup in
  autogen)
    oc -n $ADEI_NS new-app --template=adei -p setup=autogen -p enable_logs=0 -p adei_replicas=1 -p cache_replicas=1 -p sched_parallel=sources -p cache_parallel=sources -p continuous_caching=1
    ;;
  katrin)
    oc -n $ADEI_NS new-app --template=adei -p setup=katrin -p enable_logs=1 -p adei_replicas=1 -p cache_replicas=3 -p sched_parallel=sources -p cache_parallel=groups -p haproxy_timeout=300s -p maintain_schedule="*/17 * * * *" -p update_schedule="*/7 * * * *"
    ;;
  darwin)
    oc -n $ADEI_NS new-app --template=adei -p setup=darwin -p enable_logs=0 -p adei_replicas=1 -p cache_replicas=1 -p sched_parallel=sources -p cache_parallel=groups -p haproxy_timeout=300s
    ;;
  smartgrid)
    oc -n $ADEI_NS new-app --template=adei -p setup=smartgrid -p enable_logs=0 -p adei_replicas=1 -p cache_replicas=1 -p sched_parallel=sources -p cache_parallel=groups -p haproxy_timeout=300s
    ;;
  asec)
    oc -n $ADEI_NS new-app --template=adei -p setup=asec -p enable_logs=0 -p adei_replicas=1 -p cache_replicas=1 -p sched_parallel=sources -p cache_parallel=sources -p continuous_caching=1
    ;;
  munin)
    oc -n $ADEI_NS new-app --template=adei -p setup=munin -p data_path=munin -p forbid_data_writes=false -p enable_logs=0 -p adei_replicas=1 -p cache_replicas=1 -p archive_replicas=0 -p sched_parallel=servers -p cache_parallel=servers -p haproxy_timeout=300s
    oc -n $ADEI_NS process munin-builders | oc -n $ADEI_NS create -f -
    oc -n $ADEI_NS new-app --template=munin-pods -p munin_nodes="styx:192.168.26.117"
    ;;
  all)
    bash provision.sh -s autogen "$@"
    bash provision.sh -s katrin "$@"
    bash provision.sh -s darwin "$@"
    bash provision.sh -s smartgrid "$@"
    ;;
  *)
    echo "setup is not configured yet"
esac
