#! /bin/bash

run=1
trap 'run=0' INT

mode="current"
while getopts "h:m:" arg; do
  case $arg in
    h)
      echo "$0 [-m mode]" 
      ;;
    m)
      mode=$OPTARG
      ;;
  esac
done



ADEI_PATH=${ADEI_PATH:-/srv/www/htdocs/adei}
ADEI_URL=${ADEI_URL:-http://localhost/adei}
ADEI_PARALLEL=${ADEI_PARALLEL:-sources}
ADEI_IDLE_SLEEP=${ADEI_IDLE_SLEEP:-5}
ADEI_ERROR_SLEEP=${ADEI_ERROR_SLEEP:-1}

if [ -z "$ADEI_SETUP" ]; then
    ADEI_SETUP=$(cd "$ADEI_PATH"; php  -r 'require "adei.php"; echo $ADEI_SETUP;')
fi

mode_arg=""
[ -n "$mode" ] && mode_arg="mode=${mode}&"

ADEI_LOCK_PATH="${ADEI_PATH}/tmp/locks/";
ADEI_SCHEDULER_URL="${ADEI_SCHEDULER_URL:-$ADEI_URL/services/sched.php?setup=${ADEI_SETUP}&target=get&action=cache&${mode_arg}encoding=text}"

shopt -s nocasematch

while [ $run -eq 1 ]; do
    src=$(curl -sf $ADEI_SCHEDULER_URL)
    err=$?
    
    date=$(date -Iseconds)
    if [ $err -ne 0 ]; then
        err=$(curl -f $ADEI_SCHEDULER_URL 2>&1)
        echo "$date Scheduling error at $ADEI_SCHEDULER_URL"
        echo "$date $err"
        sleep "$ADEI_ERROR_SLEEP"
    elif [ -d /adei/cfg -a ! -f "/adei/cfg/.ready" ]; then
        echo "$date ADEI is not ready yet. Idling..."
        sleep "$ADEI_IDLE_SLEEP"
    elif [ ! -f "$ADEI_PATH/config.php" -o ! -e "$ADEI_PATH/config.actual.php" -o ! -f "$ADEI_PATH/setups/$ADEI_SETUP/config.php" ]; then
        echo "$date ADEI is not properly configured. Waiting..."
        sleep "$ADEI_IDLE_SLEEP"
    elif [ -f "/adei/cfg/.debug" -o -f "$ADEI_PATH/setups/$ADEI_SETUP/.debug" ]; then
        echo "$date Setup is under debug. Idling..."
        sleep "$ADEI_IDLE_SLEEP"
    elif [[ "$src" =~ ^Error ]]; then
        echo "$date Error querying scheduler service at $ADEI_SCHEDULER_URL"
        echo $src
        sleep "$ADEI_IDLE_SLEEP"
    elif [ -n "$src" ]; then
        [[ $src =~ ^([^ ]*)\ (.*)$ ]]
        name=${BASH_REMATCH[1]}
        args=${BASH_REMATCH[2]}
        md5=$(echo $args | md5sum | cut -d ' ' -f 1 )
        lock="${ADEI_LOCK_PATH}/cacher-$md5.lock"
        (
            flock -nx 10
            if [ $? -eq 0 ]; then
                args="$args -multihost -parallel ${ADEI_PARALLEL}"
                echo "$date Caching \"$name\" with arguments: $args"
                php "$ADEI_PATH/system/cache.php" $args
                [ $? -ne 0 ] && sleep "$ADEI_ERROR_SLEEP"
            else
                echo "$date Skipping \"$name\" which is processed by another cacher"
            fi
        ) 10> $lock
    elif [ $run -eq 1 ]; then
        echo "$date Idling..."
        sleep "$ADEI_IDLE_SLEEP"
    fi
done
